from typing import Dict
from abc import ABC, abstractmethod
from ..models import Tenant


class Provisioner(ABC):
    """Tenant Provision service."""

    @property
    @abstractmethod
    def location(self) -> Dict[str, str]:
        "Show provisioner data location."

    @abstractmethod
    def provision_tenant(self, tenant: Tenant) -> None:
        "Provision tenant method to be implemented."



from typing import Dict
from pathlib import Path
from shutil import copytree
from ..common import TenantProvisionError
from ..models import Tenant
from .provisioner import Provisioner


class DirectoryProvisioner(Provisioner):

    def __init__(self, template: str, data: str) -> None:
        self.template = template
        self.data = data

    @property
    def location(self) -> Dict[str, str]:
        return {
            "directory": self.data
        }

    def provision_tenant(self, tenant: Tenant) -> None:
        tenant_directory = str(Path(self.data) / tenant.slug)
        copytree(self.template, tenant_directory)



from typing import Dict
from subprocess import run, PIPE
from uuid import uuid4
from ..common import TenantProvisionError
from ..models import Tenant
from .provisioner import Provisioner


class SchemaProvisioner(Provisioner):

    def __init__(self, uri: str, template='__template__') -> None:
        self.uri = uri
        self.template = template

    @property
    def location(self) -> Dict[str, str]:
        return {
            "schema": self.uri
        }

    def provision_tenant(self, tenant: Tenant) -> None:
        command = (
            f"set -o pipefail; "
            f"pg_dump {self.uri} --schema={self.template} | "
            f"sed 's/{self.template}/{tenant.slug}/g' | "
            f"psql {self.uri}")
        run(command, shell=True, check=True, stdout=PIPE, stderr=PIPE)



from typing import Dict
from ..common import TenantProvisionError
from ..models import Tenant
from .provisioner import Provisioner


class MemoryProvisioner(Provisioner):

    def __init__(self) -> None:
        self.pool: Dict[str, Tenant] = {}

    @property
    def location(self) -> Dict[str, str]:
        return {
            "memory": ""
        }

    def provision_tenant(self, tenant: Tenant) -> None:
        self.pool[tenant.id] = tenant



from abc import ABC, abstractmethod
from typing import List, Dict, Any, Optional
from ..common import QueryParser, QueryDomain, TenantRetrievalError
from ..models import Tenant
from .cataloguer import Cataloguer


class MemoryCataloguer(Cataloguer):

    def __init__(self, parser: QueryParser) -> None:
        self.parser = parser
        self.catalog: Dict[str, Tenant] = {}

    def add_tenant(self, tenant: Tenant) -> Tenant:
        self.catalog[tenant.id] = tenant
        return tenant

    def get_tenant(self, tenant_id: str) -> Tenant:
        tenant = self.catalog.get(tenant_id)

        if not tenant:
            raise TenantRetrievalError('Tenant not found.')

        return tenant

    def search_tenants(self, domain: QueryDomain) -> List[Tenant]:
        tenants = []
        filter_function = self.parser.parse(domain)
        for tenant in self.catalog.values():
            if filter_function(tenant):
                tenants.append(tenant)

        return tenants



from .provider import Provider
from .arranger import Arranger
from .resolver import (
    resolve_arranger, resolve_cataloguer,
    resolve_provider, resolve_provisioner,
    resolve_identifier)

__version__ = '0.2.3'
